<script setup lang="ts">
import EditorWorker from "monaco-editor/esm/vs/editor/editor.worker?worker";
import JSONWorker from "monaco-editor/esm/vs/language/json/json.worker?worker";
import CSSWorker from "monaco-editor/esm/vs/language/css/css.worker?worker";
import HTMLWorker from "monaco-editor/esm/vs/language/html/html.worker?worker";
import TSWorker from "monaco-editor/esm/vs/language/typescript/ts.worker?worker";
import { Splitpanes, Pane } from "splitpanes";
import "splitpanes/dist/splitpanes.css";

import Editor from "./Editor.vue";
import { LANGUAGES, type PAYLOAD } from "@/types";
import { ref } from "vue";
import { generateHTML } from "@/utils";

let htmlCode = ref(`
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Your Landing Page</title>
    <style>
        body {
            margin: 0;
            padding: 0;
            display: flex;
            justify-content: center;
            align-items: center;
            height: 100vh;
        }
        
        .container {
            text-align: center;
        }

        h1 {
            font-size: 36px;
            margin-bottom: 20px;
        }
    </style>
</head>
<body>
    <div class="container">
        <img src="https://zahidaz.com/assets/img/logo-zahidaz.png" alt="Welcome Image" width="100px">
        <h1>Welcome to</h1>
        <p>Html Editor</p>
    </div>
</body>
</html>
`);
let cssCode = ref("");
let jsCode = ref("");

let iframe = ref<HTMLIFrameElement>();


const orientation = ref('vertical');

const toggleOrientation = () => {
  orientation.value = orientation.value === 'vertical' ? 'horizontal' : 'vertical';
};

const saveToServer = () => {
  const code = htmlCode.value;
  // console.log(code);
  alert('available soon');
  // show bootstrap modal
  // ask login
  // ask title
  // ask author name
  // ask credit
  // ask credit url
};


const downloadAsHtml = () => {
  const blob = new Blob([htmlCode.value], { type: 'text/html' });
  const url = URL.createObjectURL(blob);
  const a = document.createElement('a');
  a.href = url;
  a.download = 'content.html';
  document.body.appendChild(a);
  a.click();
  document.body.removeChild(a);
  URL.revokeObjectURL(url);
};

function onChange(payload: PAYLOAD) {
  if (payload.type === LANGUAGES.HTML) {
    htmlCode.value = payload.code;
  } else if (payload.type === LANGUAGES.CSS) {
    cssCode.value = payload.code;
  } else if (payload.type === LANGUAGES.JS) {
    jsCode.value = payload.code;
  }

  iframe.value!.srcdoc = generateHTML({
    html: htmlCode.value,
    css: cssCode.value,
    js: jsCode.value,
  });
}

self.MonacoEnvironment = {
  getWorker(_: string, label: string) {
    if (label === "json") return new JSONWorker();

    if (label === "css" || label === "scss" || label === "less")
      return new CSSWorker();

    if (label === "html" || label === "handlebars" || label === "razor")
      return new HTMLWorker();

    if (label === "typescript" || label === "javascript") return new TSWorker();

    return new EditorWorker();
  },
};
</script>
<template>
  <main>
    <div class="flex">
      <a class="bg-gray-300 ps-1 py-2" href="https://zahidaz.com">
        <img class="me-2" src="https://zahidaz.com/assets/img/logo-zahidaz.png" width="25px"/>
      </a>
      <button class="bg-gray-300 px-3 py-2" @click="downloadAsHtml">
        <i class="fas fa-download"></i>
      </button>
      <button class="bg-gray-300 px-3 py-2" @click="toggleOrientation">
        <i class="fas fa-sync"></i>
      </button>
      <button class="bg-gray-300 px-3 py-2" @click="saveToServer">
        <i class="fas fa-cloud-upload-alt"></i>
      </button>
    </div>
    <div style="overflow:auto; height:100%">
      <Splitpanes class="w-full h-full"  :horizontal="orientation === 'horizontal'">
        <Pane min-size="20" class="border-b border-gray-700">
          <Editor
            class="w-full h-full"
            :type="LANGUAGES.HTML"
            :display-name="LANGUAGES.HTML"
            @code-change="onChange"
            sandbox="allow-modals"
          />
          <!--Splitpanes vertical>
            <Pane min-size="5">
              <Editor
                class="w-full h-full"
                :type="LANGUAGES.HTML"
                :display-name="LANGUAGES.HTML"
                @code-change="onChange"
              />
            </Pane>

            <Pane min-size="5">
              <Editor
                class="w-full h-full"
                :type="LANGUAGES.CSS"
                :display-name="LANGUAGES.CSS"
                @code-change="onChange"
              />
            </Pane>
            <Pane min-size="5">
              <Editor
                class="w-full h-full"
                :type="LANGUAGES.JS"
                :display-name="LANGUAGES.shortJs"
                @code-change="onChange"
              />
            </Pane>
          </Splitpanes-->
        </Pane>
        <Pane
          ><iframe
            class="w-full h-full bg-white border ms-2"
            ref="iframe"
            sandbox="allow-scripts allow-modals"
            frameborder="0"
          ></iframe
        ></Pane>
      </Splitpanes>
    </div>
  </main>
</template>
<style>
main {
  height: calc(100vh - var(--nav-height));
}

.splitpanes--horizontal > .splitpanes__splitter {
  min-height: 0.5rem;
  background: black;
}

.splitpanes--vertical > .splitpanes__splitter {
  min-width: 0.5rem;
  background: black;
}
</style>
